;**********************************************************************
;Program 8-chanal timer controller
;                                                                     *
;    Filename:	    timer.asm                                       *
;    Date:                                                            *
;    File Version:                                                    *
;                                                                     *
;    Author:                                                          *
;    Company:                                                         *
;                                                                     * 
;                                                                     *
;**********************************************************************
;                                                                     *
;    Notes:                                                           *
; PA0-PA2 - select digit, output
; PA3	  - keyboard input
; PA4-PA5 - I2C bus line SDA,SCL                                                                    *
; PC0-PC7 - LED segment,  output                                                                  *
; PB0-PB7 - Signal output                                                                    *
;                                                                     *
;**********************************************************************


	list      p=pic16f876            ; list directive to define processor
	#include <p16f876.inc>        ; processor specific variable definitions

;#define debug
	
	__CONFIG _CP_OFF & _WDT_ON & _BODEN_ON & _PWRTE_ON & _HS_OSC & _WRT_ENABLE_OFF & _LVP_OFF & _DEBUG_OFF & _CPD_OFF 

; '__CONFIG' directive is used to embed configuration data within .asm file.
; The lables following the directive are located in the respective .inc file.
; See respective data sheet for additional information on configuration word.

#define	_C		STATUS,0
#define	_Z		STATUS,2

delay	EQU	.255-.140
tmr_work EQU	.300

;***** VARIABLE DEFINITIONS
w_temp		EQU     0x70        ; variable used for context saving 
status_temp	EQU     0x71        ; variable used for context saving
PCLATH_TEMP	EQU		0x72
fsr_temp	EQU		0x73

	cblock	0x20
segment:	6	;screen data
dispcnt:	1
f_data:		1	;flashing position
f_count:	1
fmask1:		1
fmask2:		1
f_timl:		1
f_timh:		1
cond:		1
mode:		1
time_out:	1
hour:		1
minute:		1
adr_min:	1
adr_hour:	1
w_tmrl:		1
w_tmrh:		1
rom_dat:	1
key_stat:	1
key_old:	1
key_tim:	1
key_num:	1
key_rep:	1
				;i2c communication
FLAG:    	1      ; Common flag bits register
EEPROM:  	1      ; Bit buffer
ERCODE:  	1      ; Error code (to indicate bus status)
ADDR:    	1      ; Address register
DATAI:   	1      ; Stored data input register
DATAO:   	1      ; Stored data output register
SLAVE:   	1      ; Device address (1010xxx0)
TXBUF:   	1      ; TX buffer
RXBUF:		1      ; RX buffer
;i2c_tim:	1		;delay timer
				;math data
H_byte:		1
L_byte:		1
R0:			1
R1:			1
R2:			1
temp:		2
count:  	1

	endc

;bitfield definition

#define timer_ready	cond,0	;timer interrupt was accur
#define	flash_state	cond,1
#define	busy		cond,2
#define	read_req	cond,3
#define	key_repeat	cond,4
#define wrk_tim		cond,5

#define key0	PORTA,3
#define	SDA		PORTA,5
#define	SCL		PORTA,4
#define	SDA_TRIS TRISA,5
#define	SCL_TRIS TRISA,4

ERR1    EQU     0               ; Error flag
;
; Instruction Destination Bits
;
;
; EEPROM Bits
;
DI      EQU     7               ; EEPROM input
DO      EQU     6               ; EEPROM output


;**********************************************************************
	ORG     0x000           ; processor reset vector
	clrf    PCLATH          ; ensure page bits are cleared
  	goto    init            ; go to beginning of program

	ORG     0x004           ; interrupt vector location
	movwf   w_temp          ; save off current W register contents
	movf	STATUS,w        ; move status register into W register
	movwf	status_temp     ; save off contents of STATUS register
	MOVF	PCLATH, W		;Only required if using pages 1, 2 and/or 3
	MOVWF	PCLATH_TEMP
	movf	FSR,w
	movwf	fsr_temp

	movlw	delay
	movwf	TMR0
	bcf		INTCON,T0IF
	bsf		timer_ready
	clrf	PORTC
	incf	PORTA,1
	movlw	7
	andwf	PORTA,w
	sublw	6	
	btfss	_Z
	goto	int1
	movf	f_data,w
	movwf	f_count
	movlw	0xf8
	andwf	PORTA,1	
	movlw	6
	movwf	dispcnt
int1
	decf	dispcnt,1
	movlw	segment
	addwf	dispcnt,w
	movwf	FSR
	movf	INDF,w
	btfss	PORTA,2			;direct show for status LED
	call	dec2seg
	movwf	PORTC
	btfsc	key_repeat
	goto	end_flash
	rlf		f_count,1
	btfss	_C
	goto	end_flash
	btfss	flash_state
	goto	end_flash
	clrf	PORTC
	btfsc	PORTA,2
	comf	PORTC,1
end_flash
	btfsc	PORTA,2
	goto	end_int
	movf	mode,w			;show point for input mode
	btfss	_Z
	bsf		PORTC,0
end_int
	MOVF	PCLATH_TEMP, W 	;Restore PCLATH
	MOVWF	PCLATH 			;Move W into PCLATH
	movf	fsr_temp,w
	movwf	FSR
	movf    status_temp,w	; retrieve copy of STATUS register
	movwf	STATUS          ; restore pre-isr STATUS register contents
	swapf   w_temp,f
	swapf   w_temp,w        ; restore pre-isr W register content
	retfie
                    ; return from interrupt
dec2seg
		andlw	0xF
		addwf	PCL,1
		retlw	0xde
		retlw	0x82
		retlw	0xec
		retlw	0xe6
		retlw	0xb2
		retlw	0x76
		retlw	0x7e
		retlw	0xc2
		retlw	0xfe
		retlw	0xf6
		retlw	0x1
		retlw	0x1
		retlw	0x1
		retlw	0x1
		retlw	0x1
		retlw	0

init	
		movlw	b'11100000'	;setup interrupt
		movwf	INTCON
		clrf	PORTA
		clrf	PORTB
		clrf	PORTC
		movlw	delay		;timer2 load
		movwf	TMR0
		banksel	OPTION_REG
		movlw	b'11010101'
		movwf	OPTION_REG	
		clrf	PIE1
		movlw	b'00000000'	;PortB PortC - output 
		movwf	TRISB		;
		movwf	TRISC
		movlw	0x06		; Configure all pins
		movwf	ADCON1		; as digital inputs
		movlw	b'00001000'	; PortA3 , - input
		movwf	TRISA
		banksel	cond
		movlw	count-segment-1
		movwf	count	
		movlw	segment
		movwf	FSR
init_clr
		clrf	INDF
		incf	FSR,1
		decfsz	count,1
		goto	init_clr
		movlw	6
		movwf	dispcnt
		call	init_fcnt
		call	clr_stat
		call	wrk_init
main	
		btfss	timer_ready
		goto	main
		bcf		timer_ready
		clrwdt
		decf	f_timl,1
		btfsc	_Z
		decf	f_timh,1
		movf	f_timl,w
		iorwf	f_timh,w
		btfss	_Z
		goto	get_time
		call	init_fcnt	;Load new value to flashing count
		movlw	2			;change flash_state
		xorwf	cond,1
		movf	time_out,w	;Check operation timeout
		btfsc	_Z
		goto	get_time
		decf	time_out,1
		btfss	_Z
		goto	get_time
		clrf	f_data		;Set main work mode on timeout
		clrf	mode
		call	clr_stat
		bcf		busy
get_time
		movf	dispcnt,w
		addwf	PCL,1
		goto	mode6
		goto	rd_data
		goto	tim_show
		goto	rd_hour
		goto	rd_minute
		goto	wrk_test

wrk_test
		btfss	wrk_tim
		goto	input
		decf	w_tmrl,1
		btfsc	_Z
		decf	w_tmrh,1
		movf	w_tmrl,w
		iorwf	w_tmrh,w
		btfss	_Z
		goto	input
		clrf	PORTB	;clear output on end of wrk_time
		bcf		wrk_tim
		call	wrk_init
		btfss	busy
		call	clr_stat
		goto	input

rd_minute
		movlw	0xD0
		movwf	SLAVE
		movlw	1
		movwf	ADDR
		call	RDBYTE
		andlw	0x7F
		subwf	minute,w;test change minute
		btfss	_Z
		bsf		wrk_tim	;set wrk_time	
rdm1	movf	DATAI,w
		movwf	minute
		btfsc	busy
		goto	input
		movf	minute,w
		movwf	R2
		goto	input

rd_hour
		movlw	0xD0
		movwf	SLAVE
		movlw	2
		movwf	ADDR
		call	RDBYTE
		andlw	0x3F
		movwf	hour
		btfss	busy
		movwf	R1
		goto	input
tim_show
		movlw	segment+5
		movwf	FSR
		call	unpack	
		goto	input
rd_data
		btfss	wrk_tim	
		goto	input
		movf	hour,w
		call	bcd2bin
		movwf	H_byte
		movf	minute,w
		call	bcd2bin
		movwf	L_byte
		call	set_adr
		call	RDBYTE
		btfsc	wrk_tim	;skip if wrk_time=0
		movwf	PORTB
		btfsc	busy	;skip out data show on input mode
		goto	input
		comf	PORTB,w
		movwf	segment
input						;read keyboard 
		movf	dispcnt,w
		btfss	_Z
		goto	keyscan
		movf	key_stat,w
		btfss	_Z
		goto	k_press
		movwf	key_old
		movlw	.5
		movwf	key_tim
		bcf		key_repeat
		goto	main
k_press	movf	key_tim,w
		btfsc	_Z
		goto	k_ready
		decf	key_tim,1
		goto	main

keyscan	btfsc	key0		;scan keypad
		goto	main
		movf	dispcnt,w
		movwf	key_stat
		goto	main

k_set	clrf	key_stat
		decfsz	key_rep,1
		goto	main
		bsf		key_repeat
		movlw	.50	;send keypress 
		movwf	key_rep
		goto	k_xmt


k_ready	movf	key_stat,w
		subwf	key_old,w
		btfsc	_Z
		goto	k_set
		movlw	.200	;set autorepeat time
		movwf	key_rep
		movlw	.150
		movwf	time_out
		movf	key_stat,w
		movwf	key_old
		movlw	2
		subwf	key_old,w
		movwf	key_num
k_xmt	movf	mode,w
		addwf	PCL,1
		goto	mode0
		goto	mode1
		goto	mode2
		goto	mode3
		goto	mode4
		goto	mode5



		org		0x100
j_main	clrf	PCLATH
		goto	main				
mode0
		movlw	high(mode0)
		movwf	PCLATH
		movf	key_num,w
		addwf	PCL,1
		goto	key_02
		goto	key_01
		goto	j_main
		goto	j_main

key_01					;Set to minute correction mode
		movlw	1
		movwf	mode
;		bsf		busy
		movlw	0x30
		movwf	f_data
		goto	j_main
key_02					;set to output correction mode
		movlw	3
		movwf	mode
		bsf		busy
		movf	adr_min,w
		movwf	R2
		clrf	R1		;clear hour position
		decf	R1,1
		movf	R1,w	;set all chanal
		movwf	segment+1
		movlw	0x30
		movwf	f_data
;		clrf	adr_min
;		clrf	adr_hour
		goto	j_main
mode1
		movlw	high(mode1)
		movwf	PCLATH
		movf	key_num,w
		addwf	PCL,1
		goto	key_13
		goto	key_12
		goto	key_10
		goto	key_11

key_10	clrf	DATAO	;Clear seconds counter
		clrf	ADDR
tmr_wr
		movlw	0xD0
		movwf	SLAVE
		call	WRBYTE
		goto	j_main

key_11					;Increment minute counter
		movlw	.1
		movwf	ADDR
		movf	minute,w
		call	inc10
		movwf	minute		
		movlw	0x60
		subwf	minute,w
		btfsc	_Z
		clrf	minute
		movf	minute,w
		movwf	DATAO
		goto	tmr_wr

key_12					;Change mode to hour set
		movlw	2
		movwf	mode
		movlw	0xC0
		movwf	f_data
		goto	j_main

key_13					;Change mode to work
		clrf	mode
		clrf	f_data
		clrf	segment+1
		comf	segment+1,1
		clrf	segment
		comf	segment,1		
		bcf		busy
		goto	j_main

mode2					;Set hour mode
		movlw	high(mode2)
		movwf	PCLATH
		movlw	.2		
		movwf	ADDR
		movf	key_num,w
		addwf	PCL,1
		goto	key_13
		goto	key_13
		goto	key_20
		goto	key_21

key_20
		movf	hour,w
		btfsc	_Z
		goto	j_main
		call	dec10	
		movwf	DATAO
		movwf	hour
		goto	tmr_wr

key_21	movf	hour,w
		call	inc10
		movwf	hour
		movlw	0x24
		subwf	hour,w
		btfsc	_Z
		clrf	hour
		movf	hour,w
		movwf	DATAO
		goto	tmr_wr

mode3
		movlw	high(mode3)
		movwf	PCLATH
		movf	key_num,w
		addwf	PCL,1
		goto	key_32
		goto	key_13
		goto	key_31
		goto	key_30
key_30		
		movf	adr_min,w
		call	inc10
		movwf	adr_min
		movlw	0x60
		subwf	adr_min,w
		btfsc	_Z
		clrf	adr_min
		movf	adr_min,w
		movwf	R2
		goto	j_main
key_31
		movf	adr_min,w
		btfsc	_Z
		goto	j_main
		call	dec10	
		movwf	adr_min
		movwf	R2
		goto	j_main
key_32
		movlw	4
		movwf	mode
		movlw	0xC0
		movwf	f_data
		movf	adr_hour,w
		movwf	R1
		goto	j_main

mode4
		movlw	high(mode4)
		movwf	PCLATH
		movf	key_num,w
		addwf	PCL,1
		goto	key_42
		goto	key_13
		goto	key_41
		goto	key_40
key_40
		movf	adr_hour,w
		call	inc10
		movwf	adr_hour
		movlw	0x24
		subwf	adr_hour,w
		btfsc	_Z
		clrf	adr_hour
		movf	adr_hour,w
		movwf	R1
		goto	j_main
key_41
		movf	adr_hour,w
		btfsc	_Z
		goto	j_main
		call	dec10	
		movwf	adr_hour
		movwf	R1
		goto	j_main
key_42
		movlw	5
		movwf	mode
		clrf	f_data
		movlw	0x7F
		movwf	segment+1
		call	set_rom
		call	RDBYTE
		movwf	rom_dat
		comf	rom_dat,w
		movwf	segment
		goto	j_main
mode5
		movlw	high(mode5)
		movwf	PCLATH
		movf	key_num,w
		addwf	PCL,1
		goto	key_52
		goto	key_13
		goto	key_50
		goto	key_51
key_50
		bsf		_C
		rrf		segment+1,1
		btfss	_C	
		bcf		segment+1,7
		goto	j_main
		
key_51
		comf	segment+1,w
		xorwf	rom_dat,1
		comf	rom_dat,w
		movwf	segment
		goto	j_main
key_52
		movf	rom_dat,w
		movwf	DATAO
		call	set_rom
		call	WRBYTE
		bsf		read_req	;wait until 24c16 writing complete
		goto	j_main

mode6
		btfss	read_req	;read new values for editing
		goto	input
		bcf		read_req
		movf	adr_min,w
		call	inc10
		movwf	adr_min
		movwf	R2
		movlw	0x60
		subwf	adr_min,w
		btfss	_Z
		goto	rd_rom
		clrf	adr_min
		clrf	R2
		movf	adr_hour,w
		call	inc10
		movwf	adr_hour
		movwf	R1
		movlw	0x24
		subwf	adr_hour,w
		btfss	_Z
		clrf	adr_hour
		clrf	R1
rd_rom
		movlw	0x7F
		movwf	segment+1
		call	set_rom
		call	RDBYTE
		movwf	rom_dat
		comf	rom_dat,w
		movwf	segment
		goto	input

;===============================
; end main body
;===============================

		org	0x200
init_fcnt
		movlw	high(.500)
		movwf	f_timh
		movlw	low(.500)
		movwf	f_timl
		return

clr_stat
		movlw	0xFF
		movwf	segment
		movwf	segment+1
		return

wrk_init
		movlw	1+high(tmr_work)
		movwf	w_tmrh
		movlw	low(tmr_work)
		movwf	w_tmrl
		return
		

inc10	movwf	temp
		incf		temp,1
		movlw	0xf
		andwf	temp,w
		sublw	0xA
		btfsc	_Z
		goto	corr_up
		movf	temp,w
		return
corr_up	movlw	0x10
		addwf	temp,w
		andlw	0xF0
		return	

dec10	movwf	temp
		andlw	0xF
		btfsc	_Z
		goto	corr_dn
		decf	temp,w
		return
corr_dn movlw	0x10
		subwf	temp,w
		iorlw	0x9
		return
;
set_rom
		movf	adr_hour,w
		call	bcd2bin
		movwf	H_byte
		movf	adr_min,w
		call	bcd2bin
		movwf	L_byte
set_adr
		rlf		L_byte,1
		rlf		L_byte,1
		rrf		H_byte,1
		rrf		L_byte,1
		rrf		H_byte,1
		rrf		L_byte,1
		bcf		_C
		rlf		H_byte,w
		iorlw	0xA0
		movwf	SLAVE
		movf	L_byte,w
		movwf	ADDR
		return


;
;subroutine

;delete leading zero
;
;del_zero
;	movf	INDF,w
;	btfss	_Z
;	return
;	movlw	0xf
;	movwf	INDF
;	incf	FSR,1
;	decfsz	count,1
;	goto	del_zero
;	return
;

;decimal to binary conversion
bcd2bin
	movwf	temp+1
	swapf   temp+1,w
	andlw   0F
	movwf   L_byte
	bcf     STATUS,C        ; multiply by 2
	rlf     L_byte,W
	movwf   temp			; temp = 2*a
	bcf     STATUS,C        ; L_byte multiply by 2
	rlf     L_byte, F
	bcf     STATUS,C        ; L_byte multiply by 2
	rlf     L_byte, F
	bcf     STATUS,C        ; L_byte multiply by 2
	rlf     L_byte, F		; L_byte = 8*a
;
	movf    temp,W			 
	addwf   L_byte,1
	movf	temp+1,w
	andlw	0xF
	addwf	L_byte,w
	return					; w = 10*a+b
;
;
;convert 10-bit adres to I2C command 24C16
; in	H_byte,L_byte - adres
; out	ADDR,SLAVE - result
;

;
;binary to decimal line conversion
;
b2_bcd
	bcf     _C                ; clear the carry bit
	movlw   .16
	movwf   count
	clrf    R0
	clrf    R1
	clrf    R2
loop16
	rlf     L_byte, F
	rlf     H_byte, F
	rlf     R2, F
	rlf     R1, F
	rlf     R0, F
;
	decfsz  count, F
	goto    adjDEC
	RETLW   0
;
adjDEC  movlw   R2
	movwf   FSR
	call    adjBCD
;
	movlw   R1
	movwf   FSR
	call    adjBCD
;
	movlw   R0
	movwf   FSR
	call    adjBCD
;
	goto    loop16
;
adjBCD  movlw   3
	addwf   0,W
	movwf   temp
	btfsc   temp,3          ; test if result > 7
	movwf   0
	movlw   30
	addwf   0,W
	movwf   temp
	btfsc   temp,7          ; test if result > 7
	movwf   0               ; save as MSD
	RETLW   0

unpack
	movlw	2
	movwf	count
	movf	R1,w
unp1
	movwf	temp
	andlw	0xf0
	movwf	INDF
	bcf		_C
	rrf		INDF,1
	rrf		INDF,1
	rrf		INDF,1
	rrf		INDF,1
	decf	FSR,1
	movlw	0xf
	andwf	temp,w
	movwf	INDF
	decf	FSR,1
	movf	R2,w
	decfsz	count,1
	goto	unp1
	return

;-----------------------------------------------------------------------------
;       Two wire/I2C - CPU communication error status table and subroutine
;-----------------------------------------------------------------------------
;  input  :     W-reg   = error code
;  output :     ERCODE  = error code
;           FLAG(ERR1) = 1
;
;         code          error status mode
;       -------         ------------------------------------------------------
;           1   :       SCL locked low by device (bus is still busy)
;           2   :       SDA locked low by device (bus is still busy)
;           3   :       No acknowledge from device (no handshake)
;           4   :       SDA bus not released for master to generate STOP bit
;-----------------------------------------------------------------------------
;
;Subroutine to identify the status of the serial clock (SCL) and serial data
;(SDA) condition according to the error status table. Codes generated are
;useful for bus/device diagnosis.
;
ERR
	BTFSS   FLAG,ERR1       ; Remain as first error encountered
	MOVWF   ERCODE          ; Save error code
	BSF     FLAG,ERR1       ; Set error flag
	RETLW   0

i2c_wait
	return
;
;-----------------------------------------------------------------------------
;       START bus communication routine
;-----------------------------------------------------------------------------
;       input   : none
;       output  : initialize bus communication
;-----------------------------------------------------------------------------
;
;Generate START bit (SCL is high while SDA goes from high to low transition)
;and check status of the serial clock.
BSTART
	bsf		STATUS,RP0
	bcf		SDA_TRIS     ; Put  SDA line in output state
	bcf		SCL_TRIS
	bcf		STATUS,RP0
;***************************************************************************
	bsf     SDA          ;make sure sda is high
;***************************************************************************
	BSF     SCL          ; Set clock high
	call	i2c_wait
	call	i2c_wait
	MOVLW   1               ; Ready error status code 1
	BTFSS   SCL          ; Locked?
	CALL    ERR          ; SCL locked low by device
	BCF     SDA          ; SDA goes low during SCL high
	call	i2c_wait         ; Timing adjustment
	call	i2c_wait
	call	i2c_wait
	BCF     SCL          ; Start clock train
	call	i2c_wait
	RETLW   0
;
;-----------------------------------------------------------------------------
;       STOP bus communication routine
;-----------------------------------------------------------------------------
;       Input   :       None
;       Output  :       Bus communication, STOP condition
;-----------------------------------------------------------------------------
;
;Generate STOP bit (SDA goes from low to high during SCL high state)
;and check bus conditions.
;
BSTOP
;****************************************************************************
	bsf		STATUS,RP0
	bcf		SDA_TRIS     ; Put SCL, SDA line in output state
	bcf		SCL_TRIS
	bcf		STATUS,RP0
;****************************************************************************
	BCF     SDA          ; Return SDA to low
	BSF     SCL          ; Set SCL high
	call	i2c_wait	;nop
	call	i2c_wait	;nop
	call	i2c_wait	;nop
	MOVLW   1               ; Ready error code 1
	BTFSS   SCL          ; High?
	CALL    ERR             ; No, SCL locked low by device
	BSF     SDA          ; SDA goes from low to high during SCL high
	MOVLW   4               ; Ready error code 4
	BTFSS   SDA          ; High?
	CALL    ERR             ; No, SDA bus not release for STOP
	RETLW   0
;
;END SUB
;
;-----------------------------------------------------------------------------
;       Serial data send from PIC to serial EEPROM, bit-by-bit subroutine
;-----------------------------------------------------------------------------
;       Input   :       None
;       Output  :       To (DI) of serial EEPROM device
;-----------------------------------------------------------------------------
;
BITIN
	bsf		STATUS,RP0
	bsf		SDA_TRIS     ; Force SDA line as input
	bcf		STATUS,RP0
	BSF     SDA          ; Set SDA for input
	BCF     EEPROM,DI
	BSF     SCL          ; Clock high
	MOVLW   1
	BTFSC   SCL          ; Skip if SCL  is high
	GOTO    BIT1
	BTFSS   FLAG,ERR1       ; Remain as first error encountered
	MOVWF   ERCODE          ; Save error code
	BSF     FLAG,ERR1       ; Set error flag
BIT1
	call	i2c_wait
	call	i2c_wait
	call	i2c_wait
	BTFSC   SDA          ; Read SDA pin
	BSF     EEPROM,DI       ; DI = 1
	call	i2c_wait
	call	i2c_wait        ; Delay
	BCF     SCL          ; Return SCL to low     
	RETLW   0
;
;END SUB
 PAGE
;
;-----------------------------------------------------------------------------
;       Serial data receive from serial EEPROM to PIC, bit-by-bit subroutine
;-----------------------------------------------------------------------------
;       Input   :       EEPROM file
;       Output  :       From (DO) of serial EEPROM device to PIC
;-----------------------------------------------------------------------------
;
BITOUT
	bsf		STATUS,RP0	
	bcf		SDA_TRIS     ; Set SDA, SCL as outputs
	bcf		STATUS,RP0
	BTFSS   EEPROM,DO
	GOTO    BIT0
	BSF     SDA          ; Output bit 0
	MOVLW   2
	BTFSC   SDA          ; Check for error code 2
	GOTO    CLK1
	BTFSS   FLAG,ERR1       ; Remain as first error encountered
	MOVWF   ERCODE          ; Save error code
	BSF     FLAG,ERR1       ; Set error flag
	GOTO    CLK1            ; SDA locked low by device
;
BIT0
	BCF     SDA          ; Output bit 0
	NOP                     ; Delay
	call	i2c_wait
;	call	i2c_wait
;	call	i2c_wait
CLK1
	BSF     SCL
	MOVLW   1               ; Error code 1
	BTFSC   SCL          ; SCL locked low?
	GOTO    BIT2            ; No.
	BTFSS   FLAG,ERR1       ; Yes.
	MOVWF   ERCODE          ; Save error code
	BSF     FLAG,ERR1       ; Set error flag
BIT2
;	call	i2c_wait
	call	i2c_wait
	BCF     SCL          ; Return SCL to low
	RETLW   0
;
;END SUB
 PAGE
;
;
;-----------------------------------------------------------------------------
;       RECEIVE         DATA    subroutine
;-----------------------------------------------------------------------------
;       Input   :       None
;       Output  :       RXBUF = Receive 8-bit data
;-----------------------------------------------------------------------------
;
RX
	MOVLW   .8              ; 8 bits of data
	MOVWF   count
	CLRF    RXBUF
;
RXLP
	RLF     RXBUF, F        ; Shift data to buffer
	SKPC
	BCF     RXBUF,0         ; carry ---> f(0)
	SKPNC
	BSF     RXBUF,0
	CALL    BITIN
	BTFSC   EEPROM,DI
	BSF     RXBUF,0         ; Input bit =1
	DECFSZ  count, F        ; 8 bits?
	GOTO    RXLP
	BSF     EEPROM,DO       ; Set acknowledge bit = 1
	CALL    BITOUT          ; to STOP further input
	RETLW   0
;
;END SUB
;
;-----------------------------------------------------------------------------
;       TRANSMIT        DATA    subroutine
;-----------------------------------------------------------------------------
;       Input   :       TXBUF
;       Output  :       Data X'mitted to EEPROM device
;-----------------------------------------------------------------------------
;
TX
	MOVLW   .8
	MOVWF   count
;
TXLP
	BCF     EEPROM,DO       ; Shift data bit out.
	BTFSC   TXBUF,7         ; If shifted bit = 0, data bit = 0
	BSF     EEPROM,DO       ; Otherwise data bit = 1
	CALL    BITOUT          ; Serial data out
	RLF     TXBUF, F        ; Rotate TXBUF left
	SKPC                    ; f(6) ---> f(7)
	BCF     TXBUF,0         ; f(7) ---> carry
	SKPNC                   ; carry ---> f(0)
	BSF     TXBUF,0
	DECFSZ  count, F        ; 8 bits done?
	GOTO    TXLP            ; No.
	CALL    BITIN           ; Read acknowledge bit
	MOVLW   3
	BTFSC   EEPROM,DI       ; Check for acknowledgement
	CALL    ERR             ; No acknowledge from device
	RETLW   0

;-----------------------------------------------------------------------------
;       BYTE-WRITE, write one byte to EEPROM device
;-----------------------------------------------------------------------------
;       Input   :       DATAO= data to be written
;                       ADDR    = destination address
;                       SLAVE   = device address (1010xxx0)
;       Output  :       Data written to EEPROM device
;-----------------------------------------------------------------------------
;
;	ORG     080             ; The location for BYTE-WRITE routine can be
;                               ; assigned anywhere between (377-777) octal.
WRBYTE
	MOVF    SLAVE,W         ; Get SLAVE address
	MOVWF   TXBUF           ; to TX buffer
	CALL    BSTART          ; Generate START bit
	CALL    TX              ; Output SLAVE address
	MOVF    ADDR,W          ; Get WORD address
	MOVWF   TXBUF           ; into buffer
	CALL    TX              ; Output WORD address
	MOVF    DATAO,W         ; Move DATA 
	MOVWF   TXBUF           ; into buffer
	CALL    TX              ; Output DATA and detect acknowledgement
	CALL    BSTOP           ; Generate STOP bit
	return
;
;
;
;-----------------------------------------------------------------------------
;       BYTE-READ, read one byte from serial EEPROM device
;-----------------------------------------------------------------------------
;       Input   :       ADDR    = source address
;                       SLAVE   = device address (1010xxx0)
;       Output  :       DATAI   = data read from serial EEPROM
;-----------------------------------------------------------------------------
;
;	ORG     0C0             ; The location for BYTE-READ routine can be
;                               ; assigned anywhere between (377-777) octal.
RDBYTE
	MOVF    SLAVE,W         ; Move SLAVE address 
	MOVWF   TXBUF           ; into buffer (R/W = 0)
	CALL    BSTART          ; Generate START bit
	CALL    TX              ; Output SLAVE address. Check ACK.
	MOVF    ADDR,W          ; Get WORD address
	MOVWF   TXBUF
	CALL    TX              ; Output WORD address. Check ACK.
	CALL    BSTART          ; START READ (if only one device is
	MOVF    SLAVE,W         ; connected to the I2C bus)
	MOVWF   TXBUF
	BSF     TXBUF,0         ; Specify READ mode (R/W = 1)
	CALL    TX              ; Output SLAVE address
	CALL    RX              ; READ in data and acknowledge
	CALL    BSTOP           ; Generate STOP bit
	MOVF    RXBUF,W         ; Save data from buffer
	MOVWF   DATAI           ; to DATAI file.
	return

	end
